@@ -5,7 +5,7 @@ package DBIx::SearchBuilder;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "1.11";
+$VERSION = "1.12";
 
 =head1 NAME
 
@@ -74,6 +74,7 @@ sub CleanSlate {
     $self->{'order'}            = "";
     $self->{'alias_count'}      = 0;
     $self->{'first_row'}        = 0;
+    $self->{'must_redo_search'} = 1;
     @{ $self->{'aliases'} } = ();
 
     delete $self->{'items'}        if ( defined $self->{'items'} );
@@ -120,13 +121,13 @@ sub _DoSearch {
         # TODO: finer-grained eval and cheking.
        my  $records = $self->_Handle->SimpleQuery($QueryString);
         my $counter;
+        $self->{'rows'} = 0;
         while ( my $row = $records->fetchrow_hashref() ) {
-            $self->{'items'}[$counter] = $self->NewItem();
-            $self->{'items'}[$counter]->LoadFromHash($row);
-            $counter++;
+            my $item = $self->NewItem();
+            $item->LoadFromHash($row);
+            $self->AddRecord($item);
         }
 
-        $self->{'rows'} = $counter;
         $self->{'must_redo_search'} = 0;
     };
 
@@ -135,6 +136,20 @@ sub _DoSearch {
 
 # }}}
 
+=head2 AddRecord RECORD
+
+    Adds a record object to this collection
+
+=cut
+
+sub AddRecord {
+    my $self = shift;
+    my $record = shift;
+   push @{$self->{'items'}}, $record;
+   $self->{'rows'}++; 
+}
+
+
 # {{{ sub _DoCount
 
 sub _DoCount {
@@ -194,7 +209,7 @@ sub _DistinctQuery {
     my $table = shift;
 
     # XXX - Postgres gets unhappy with distinct and OrderBy aliases
-    if ($self->{order_clause} =~ /(?<!main)\./) {
+    if (exists $self->{'order_clause'} && $self->{'order_clause'} =~ /(?<!main)\./) {
         $$statementref = "SELECT main.* FROM $$statementref";
     }
     else {
