@@ -6,6 +6,7 @@ use strict;
 use Class::ReturnValue;
 use vars qw($VERSION @ISA %DBIHandle $PrevHandle $DEBUG $TRANSDEPTH);
 
+
 $TRANSDEPTH = 0;
 
 $VERSION = '$Version$';
@@ -57,6 +58,8 @@ sub new  {
     my $class = ref($proto) || $proto;
     my $self  = {};
     bless ($self, $class);
+
+    @{$self->{'StatementLog'}} = ();
     return $self;
 }
 
@@ -233,6 +236,67 @@ sub PrintError {
 
 # }}}
 
+=head2 LogSQLStatements BOOL
+
+Takes a boolean argument. If the boolean is true, SearchBuilder will log all SQL
+statements, as well as their invocation times and execution times.
+
+Returns whether we're currently logging or not as a boolean
+
+=cut
+
+sub LogSQLStatements {
+    my $self = shift;
+    if (@_) {
+
+        require Time::HiRes;
+    $self->{'_DoLogSQL'} = shift;
+    return ($self->{'_DoLogSQL'});
+    }
+}
+
+=head2 _LogSQLStatement STATEMENT DURATION
+
+add an SQL statement to our query log
+
+=cut
+
+sub _LogSQLStatement {
+    my $self = shift;
+    my $statement = shift;
+    my $duration = shift;
+    push @{$self->{'StatementLog'}} , ([Time::Hires::time(), $statement, $duration]);
+
+}
+
+=head2 ClearSQLStatementLog
+
+Clears out the SQL statement log. 
+
+
+=cut
+
+sub ClearSQLStatementLog {
+    my $self = shift;
+    @{$self->{'StatementLog'}} = ();
+}   
+
+
+=head2 SQLStatementLog
+
+Returns the current SQL statement log as an array of arrays. Each entry is a triple of 
+
+(Time,  Statement, Duration)
+
+=cut
+
+sub SQLStatementLog {
+    my $self = shift;
+    return  (@{$self->{'StatementLog'}});
+
+}
+
+
 # {{{ AutoCommit
 
 =head2 AutoCommit [MODE]
@@ -308,7 +372,6 @@ string will be inserted into the query directly rather then as a binding.
 
 =cut
 
-## Please see file perltidy.ERR
 sub UpdateRecordValue {
     my $self = shift;
     my %args = ( Table         => undef,
@@ -378,7 +441,10 @@ Execute the SQL string specified in QUERY_STRING
 sub SimpleQuery  {
     my $self = shift;
     my $QueryString = shift;
-    my @bind_values = (@_);
+    my @bind_values;
+   @bind_values = (@_) if (@_);;
+
+    
 
     my $sth = $self->dbh->prepare($QueryString);
     unless ($sth) {
@@ -406,8 +472,20 @@ sub SimpleQuery  {
             $sth->bind_param($bind_idx+1, undef, $bhash );
         }
     }
-    $self->Log($QueryString. " (".join(',',@bind_values).")") if ($DEBUG);
-    unless ( $sth->execute(@bind_values) ) {
+
+    my $basetime;
+    if ($self->LogSQLStatements) {
+        $basetime = Time::HiRes::time(); 
+    }
+    my $executed;
+    eval {     $executed =$sth->execute(@bind_values) };
+
+    if ($self->LogSQLStatements) {
+            $self->_LogSQLStatement($QueryString ,tv_interval ( $basetime ));
+ 
+    }
+
+    if ( $@ or !$executed ) {
         if ($DEBUG) {
             die "$self couldn't execute the query '$QueryString'"
               . $self->dbh->errstr . "\n";
@@ -726,7 +804,7 @@ sub Join {
     my %args = (
         SearchBuilder => undef,
         TYPE          => 'normal',
-        FIELD1        => undef,
+        FIELD1        => 'main',
         ALIAS1        => undef,
         TABLE2        => undef,
         FIELD2        => undef,
@@ -783,10 +861,7 @@ sub Join {
 
         }
 
-        unless ($alias) {
-            return ( $self->_NormalJoin(%args) );
-        }
-        if ( $args{'ALIAS1'} ) {
+        if ( !$alias || $args{'ALIAS1'} ) {
             return ( $self->_NormalJoin(%args) );
         }
 
@@ -867,13 +942,21 @@ sub _BuildJoins {
 
     $seen_aliases{'main'} = 1;
 
+   	# We don't want to get tripped up on a dependency on a simple alias. 
+    	foreach my $alias ( @{ $sb->{'aliases'}} ) {
+          if ( $alias =~ /^(.*?)\s+(.*?)$/ ) {
+              $seen_aliases{$2} = 1;
+          }
+    }
+
     my $join_clause = $sb->{'table'} . " main ";
 
+	
     my @keys = ( keys %{ $sb->{'left_joins'} } );
     my %seen;
 
     while ( my $join = shift @keys ) {
-        if ( $seen_aliases{ $sb->{'left_joins'}{$join}{'depends_on'} } ) {
+        if ( ! $sb->{'left_joins'}{$join}{'depends_on'} || $seen_aliases{ $sb->{'left_joins'}{$join}{'depends_on'} } ) {
             $join_clause = "(" . $join_clause;
             $join_clause .=
               $sb->{'left_joins'}{$join}{'alias_string'} . " ON (";
